; tarot implements the rules of the tarot game
; Copyright (C) 2019  Vivien Kraus

; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or (at
; your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.

name "Tarot"
!define MUI_ICON "tarot.ico"

OutFile "tarot-setup.exe"
InstallDir $PROGRAMFILES64\Tarot
RequestExecutionLevel admin
; Registry key to check for directory (so if you install again, it will
; overwrite the old one automatically)
InstallDirRegKey HKLM "System\CurrentControlSet\Control\Session Manager\Environment" "TAROT_PREFIX"

!include WinMessages.nsh

!define MUI_COMPONENTSPAGE_NODESC
!include "MUI2.nsh"

Var SMDir

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_STARTMENU 0 $SMDir
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH
!insertmacro MUI_LANGUAGE "English"
!insertmacro MUI_LANGUAGE "French"

UninstPage uninstConfirm
UninstPage instfiles

Section "Tarot Program"
  SetOutPath $INSTDIR
  File /r "bin"
  File /r /x share/gir-1.0 /x share/girepository-1.0 /x share/vala /x share/dataset "share"
  File tarot.ico
  WriteRegStr HKLM "System\CurrentControlSet\Control\Session Manager\Environment" "TAROT_PREFIX" "$INSTDIR"

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "DisplayName" "Tarot"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "NoRepair" 1
  SendMessage ${HWND_BROADCAST} ${WM_WININICHANGE} 0 "STR:Environment" /TIMEOUT=5000
  WriteUninstaller "$INSTDIR\uninstall.exe"
SectionEnd

Section /o "Tarot Development files"
  SetOutPath $INSTDIR
  File /r "include"
  File /r "lib"
  File /r /x share/locale /x share/dataset "share"
  WriteRegStr HKLM "System\CurrentControlSet\Control\Session Manager\Environment" "TAROT_PREFIX" "$INSTDIR"

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "DisplayName" "Tarot"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot" "NoRepair" 1
  SendMessage ${HWND_BROADCAST} ${WM_WININICHANGE} 0 "STR:Environment" /TIMEOUT=5000
  WriteUninstaller "$INSTDIR\uninstall.exe"
SectionEnd

Section -StartMenu
  !insertmacro MUI_STARTMENU_WRITE_BEGIN 0 ;This macro sets $SMDir and skips to MUI_STARTMENU_WRITE_END if the "Don't create shortcuts" checkbox is checked... 
  CreateDirectory "$SMPrograms\$SMDir"
  CreateShortCut "$SMPROGRAMS\$SMDir\Tarot.lnk" "$INSTDIR\bin\tarot.exe" "" "$INSTDIR\tarot.ico"
  !insertmacro MUI_STARTMENU_WRITE_END
SectionEnd

Section "Desktop Shortcut"
  CreateShortCut "$DESKTOP\Tarot.lnk" "$INSTDIR\bin\tarot.exe" "" "$INSTDIR\tarot.ico"
SectionEnd

Section "Uninstall"
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\Tarot"
  DeleteRegValue HKLM "System\CurrentControlSet\Control\Session Manager\Environment" "TAROT_PREFIX"
  SendMessage ${HWND_BROADCAST} ${WM_WININICHANGE} 0 "STR:Environment" /TIMEOUT=5000

  ; Remove files and uninstaller
  RmDir /r $INSTDIR\bin
  RmDir /r $INSTDIR\include
  RmDir /r $INSTDIR\lib
  RmDir /r $INSTDIR\share
  RmDir /r "$SMPrograms\$SMDir"
  Delete $INSTDIR\uninstall.exe
  Delete $INSTDIR\tarot.ico
  Delete $DESKTOP\Tarot.lnk

  ; Remove directories used
  RMDir "$SMPROGRAMS\Tarot"
  RMDir "$INSTDIR"
SectionEnd
