/* tarot implements the rules of the tarot game
 * Copyright (C) 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "program_commons.h"
#include <tuneconfig.h>

static int generate_whole_game = 0;

static void parse_options (int argc, char *argv[]);

int
main (int argc, char *argv[])
{
  TarotGame *game = NULL;
  char *output;
  TarotPredictor *stacking = NULL;
  const TarotGameEvent *event = NULL;
  TarotGameEvent *dog_event;
  size_t n_cards;
  TarotCard *cards;
  tarot_program_setup ();
  parse_options (argc, argv);
  game = tarot_program_read_game ();
  stacking = tarot_predictor_alloc ();
  while (1)
    {
      double score;
      if (generate_whole_game)
	{
	  TarotPlayer next;
	  if (tarot_game_get_next (game, &next) != TAROT_GAME_OK)
	    {
	      break;
	    }
	}
      tarot_predictor_set_game (stacking, game);
      event = tarot_predictor_best (stacking, &score);
      if (tarot_game_add_event (game, event) != TAROT_GAME_OK)
	{
	  abort ();
	}
      if (tarot_game_can_autoreveal_alloc (game, &n_cards, &cards))
	{
	  dog_event = tarot_game_event_alloc_dog (n_cards, cards);
	  if (tarot_game_add_event (game, dog_event) != TAROT_GAME_OK)
	    {
	      abort ();
	    }
	}
      if (!generate_whole_game)
	{
	  break;
	}
    }
  output = tarot_game_save_to_xml_alloc (game);
  printf ("%s", output);
  return EXIT_SUCCESS;
}

static void
parse_options (int argc, char *argv[])
{
  int c;
  while (1)
    {
      static struct option long_options[] =
        {
	 {N_ ("help"), no_argument, 0, 'h'},
	 {N_ ("version"), no_argument, 0, 'v'},
	 {N_ ("generate"), no_argument, 0, 'g'},
	 {0, 0, 0, 0}
        };
      int option_index = 0;
      for (option_index = 0; long_options[option_index].name != NULL; option_index++)
	{
	  long_options[option_index].name = _ (long_options[option_index].name);
	}
      option_index = 0;
      c = getopt_long (argc, argv, "hvg", long_options, &option_index);
      if (c == -1)
        break;
      switch (c)
        {
        case 0:
          break;

	case 'h':
	  printf (_ ("Usage: tarot-stacking [OPTIONS]... < GAME\n\
Play as the stacking learner in GAME.  Options:\n\
- -h, --help: print this message and exit;\n\
- -v, --version: print the program version and exit;\n\
- -g, --generate: play with the same player parameters until the end\n\
   of the game.\n\
"));
	  exit (EXIT_SUCCESS);
	  break;

	case 'v':
	  printf (_ ("%s (libtarot %s)\n\
\n\
Copyright © 2019 Vivien Kraus\nThis is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
		  "tarot-stacking",
		  tarot_libtarot_version ());
	  exit (EXIT_SUCCESS);
	  break;

	case 'g':
	  generate_whole_game = 1;
	  break;

        case '?':
          /* getopt_long already printed an error message. */
	  exit (EXIT_FAILURE);
          break;

        default:
          abort ();
        }
    }
  if (optind < argc)
    {
      fprintf (stderr, _ ("Error: extra arguments:\n"));
      while (optind < argc)
        fprintf (stderr, _ ("- %s\n"), argv[optind++]);
      exit (EXIT_FAILURE);
    }
}
